/************************************************************************************
* Copyright (C) 1998 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package Message;
import java.lang.*;
import java.util.*;
import utils.*;

public class MimePart
{
	protected char _rawData[];
	protected StringBuffer _encodedData;
	protected String _type;
	protected String _subtype;
	protected String _encoding;
	protected String _name;
	protected boolean _needsEncoding;
	protected static boolean debug=false;

	public MimePart(String type, String subtype, String name, char data[]) throws MimeException
	{
		_type=type;
		_subtype=subtype;
		_name=name;
		if (!Header.Text.equals(_type))
		{
			_encoding=Header.Base64_Encoded;
			_needsEncoding=true;
		}
		_rawData=new char[data.length];
		System.arraycopy(data, 0, _rawData, 0, data.length);
	}

	public MimePart(char data[], Header header) throws MimeException
	{
		String content=header.getField(Header.Content_Type);
		if (content==null)
		{
			header.setField(Header.Content_Type, Header.Text+"/"+Header.Plain);
			content=header.getField(Header.Content_Type);
		}
		Hashtable infos=Header.parseValue(content);
		_type=(String)infos.get(Header.Type);
		_subtype=(String)infos.get(Header.Subtype);
		_name=Header.unQuote((String)infos.get(Header.Name));
		_rawData=new char[data.length];
		System.arraycopy(data, 0, _rawData, 0, data.length);
		_needsEncoding=(!Header.Text.equals(_type));
		String encoding=header.getField(Header.Content_Transfer_Encoding);
		if (encoding!=null)
		{
			_encoding=encoding.toLowerCase();
			if (!_encoding.equals(Header.Base64_Encoded))
				throw new MimeException("Don't know how to handle encoding type : "+_encoding);
		}
	}

	public MimePart(String part) throws MimeException
	{
		Header header=null;
		int idx=part.indexOf("\n\n");
		if (idx==-1)
		{
			//no header, assume Content-Type=text/plain
			header=new Header(Header.Content_Type+": text/plain\r\n");
			idx=-2; //to start composing the body at index=0
		}
		else
			header=new Header(part.substring(0, idx));
		String content=header.getField(Header.Content_Type);
		if (content==null)
			throw new MimeException("malformed part, missing content-type");
		Hashtable infos=Header.parseValue(content);
		_type=(String)infos.get(Header.Type);
		_subtype=(String)infos.get(Header.Subtype);
		_name=Header.unQuote((String)infos.get(Header.Name));
		if (Header.Text.equals(_type))
		{
			if (debug)
				System.out.println("part data starts at "+(idx+2)+" ("+part.charAt(idx+2)+")");
			String data=part.substring(idx+2);
			_rawData=data.toCharArray();
			_needsEncoding=false;
		}
		else
		{
			_encoding=header.getField(Header.Content_Transfer_Encoding);
			if (!Header.Base64_Encoded.equals(_encoding))
				throw new MimeException("Don't know how to decode "+_encoding);
			if (debug)
				System.out.println("part data starts at "+(idx+2)+" ("+part.charAt(idx+2)+"), ends at ("+part.charAt(part.length()-1)+")");
			_encodedData=new StringBuffer(part.substring(idx+2));
			_rawData=null;
			_needsEncoding=true;
		}
		if (debug)
			System.out.println("created mime part "+_type+"/"+_subtype+" encoded="+_needsEncoding);
	}
  
	public StringBuffer getEncodedData()
	{

		if (_encodedData==null)
		{
			if (needsEncoding())
			{
				if (debug)
					System.out.println("encoding part "+_name);
				_encodedData=Base64.encode(_rawData);
				_encoding=Header.Base64_Encoded;
			}
			else
			{
				StringBuffer sb=new StringBuffer(_rawData.length);
				sb.append(_rawData);
				return sb;
			}
		}
		return _encodedData;
	}

	public char[] getRawData()
	{

		if ((_rawData==null)&&needsEncoding())
		{
			if (needsEncoding())
			{
				if (debug)
					System.out.println("decoding part "+_name);
				_rawData=Base64.decode(_encodedData);
			}
			else
			{
				_rawData=new char[_encodedData.length()];
				_encodedData.getChars(0, _encodedData.length(), _rawData, 0);
			}
		}
		return _rawData;
	}

	public String getName()
	{
		return _name;
	}

	public String getEncodingType()
	{
		return _encoding;
	}

	protected boolean needsEncoding()
	{
		return _needsEncoding;
	}
  
	public String toString()
	{
		StringBuffer s=new StringBuffer(Header.Content_Type);
		s.append(": ");
		s.append(_type);
		s.append('/');
		s.append(_subtype);
		if (_name!=null)
		{
			s.append("; ");
			s.append(Header.Name);
			s.append("=\"");
			s.append(_name);
			s.append('\"');
		}
		s.append('\n');
		if (needsEncoding())
		{
			s.append(Header.Content_Transfer_Encoding);
			s.append(": ");
			if (_encoding==null)
				_encoding=Header.Base64_Encoded;
			else
				s.append(_encoding);
			s.append("\n\n");
			s.append(getEncodedData());
		}
		else
		{
			s.append('\n');
			for (int i=0; i<_rawData.length; i++)
				s.append((char)_rawData[i]);
		}
		return s.toString();
	}
}








